#!/usr/bin/env bash

#set -x

if [ $# -lt 2 ];then
    echo "usage: <rootmnt> <ostree_arg>"
    exit 1
fi

rootmnt=$1
ostree_arg=$2

if [ ! -d $rootmnt ];then
    echo "rootmnt $rootmnt not exist"
    exit 1
fi

if [ ! -d $rootmnt/persistent/$ostree_arg ];then
    echo "rootmnt $rootmnt/persistent/$ostree_arg not exist"
    exit 1
fi

ext_mount() {
    local OSTREE=$1
    local lowerdir=$2
    local datadir=$rootmnt/persistent
    local extDeploy=$(realpath $datadir/$OSTREE)
    [ ! -d $datadir/overlay/work ] && mkdir -p $datadir/overlay/work
    [ ! -d $datadir/overlay/merged ] && mkdir -p $datadir/overlay/merged
    local otDeploy=$datadir/overlay/merged
    # remove opt symlink if exists
    if [ -L "$lowerdir/opt" ]; then
        rm "$lowerdir/opt"
    fi
    mkdir -p "$lowerdir/opt"
    modprobe overlay
    mount -t overlay -o lowerdir="$lowerdir",upperdir="$extDeploy",workdir="$datadir/overlay/work" overlay $otDeploy
    mount --make-private $otDeploy
    mount -o bind,rw $otDeploy/etc $otDeploy/etc
    mount --bind $datadir/ostree/deploy/deepin/var $otDeploy/var
    mkdir -p "$otDeploy/var/mnt" || true
    mkdir -p $datadir/overlay/usr-work
    mkdir -p $datadir/overlay/usr-upper
    # Handle /opt
    mkdir -p $datadir/overlay/opt-work
    mkdir -p $datadir/overlay/opt-upper
    if [ -f $datadir/overlay/usr-upper/share/deepin-immutable-ctl/state/clean_upper ]; then
      rm -rf $datadir/overlay/usr-upper/*
      rm -rf $datadir/overlay/opt-upper/*
    fi
    mount -t overlay -o rw,relatime,lowerdir=$extDeploy/usr/opt:$lowerdir/usr/opt,upperdir=$datadir/overlay/opt-upper,workdir=$datadir/overlay/opt-work opt-overlay $otDeploy/opt
    mount -t overlay -o ro,relatime,lowerdir=$otDeploy/usr,upperdir=$datadir/overlay/usr-upper,workdir=$datadir/overlay/usr-work usr-overlay $otDeploy/usr
    mount -o bind $datadir $otDeploy/persistent
    mount -o bind $rootmnt $otDeploy/sysroot
    mount --bind -o remount,ro $otDeploy/sysroot $otDeploy/sysroot
    # chroot to roomnt and start init
    mount --make-private $rootmnt
    mount --move $otDeploy $rootmnt
}

data_layer_mount() {
    local OSTREE=$1
    local lowerdir=$2

    mount -o bind $lowerdir $lowerdir
    mount --make-private $lowerdir

    local datadir=$rootmnt/persistent
    local extDeploy=$(realpath $datadir/$OSTREE)

    # remove opt symlink if exists
    if [ -L "$lowerdir/opt" ]; then
        rm "$lowerdir/opt"
    fi

    data_commitid_idx=$(echo $extDeploy | awk -F/ '{print $(NF-1)}')
    data_overlay_root=$rootmnt/persistent/overlay/data/$data_commitid_idx
    mkdir -p $data_overlay_root
    modprobe overlay
    for mountParam in usr:ro opt:rw etc:rw;do
        dir=$(echo $mountParam | awk -F: '{print $1}')
        perm=$(echo $mountParam | awk -F: '{print $2}')
        echo "do data layer mount $dir"
        upperDir=$data_overlay_root/$dir-upper
        workDir=$data_overlay_root/$dir-work
        mkdir -p "$upperDir" "$workDir" $lowerdir/$dir

        mount -t overlay -o "$perm",relatime,lowerdir=$extDeploy/$dir:$lowerdir/$dir,upperdir=$upperDir,workdir=$workDir $dir-overlay $lowerdir/$dir
    done

    # make etc without hardlink, some apps not work well with hardlink
    hardlink_count=$(stat -c "%h" $lowerdir/etc/deepin-immutable-ctl/deepin-immutable-ctl.conf)
    if [ "$hardlink_count" -gt 1 ] && [ -f /usr/bin/cp.fix ]; then
        tmp_copyed=$rootmnt/persistent/ostree/data/tmp
        mkdir -p $tmp_copyed
        /usr/bin/cp.fix --preserve=all -r $lowerdir/etc $tmp_copyed || exit 1
        rm -rf $lowerdir/etc
        /usr/bin/cp.fix --preserve=all -r $tmp_copyed/etc $lowerdir || exit 1
        rm -rf $tmp_copyed
    fi

    rm -rf $lowerdir/media
    ln -s run/media $lowerdir/media

    mount -o bind $rootmnt $lowerdir/sysroot
    mount -o bind $lowerdir/sysroot/ostree $lowerdir/sysroot/ostree
    mount -o remount,bind,ro $lowerdir/sysroot/ostree $lowerdir/sysroot/ostree

    mount -o bind $datadir $lowerdir/persistent
    mount -o bind $lowerdir/persistent/ostree $lowerdir/persistent/ostree
    mount -o remount,bind,ro $lowerdir/persistent/ostree $lowerdir/persistent/ostree

    mount -o bind $datadir/ostree/deploy/deepin/var $lowerdir/var

    if [ -d $datadir/ostree/snapshot/next_extra ]; then
        cp -a $datadir/ostree/snapshot/next_extra/* $lowerdir
        rm -rf $datadir/ostree/snapshot/next_extra
    fi
    # chroot to roomnt and start init
    mount --make-private $rootmnt
    mount --move $lowerdir $rootmnt
}

ostree_mount_root() {
    local OSTREE=$1

    local datadir=$rootmnt/persistent

    local extDeploy=$(realpath $datadir/$OSTREE)
    local sys_commit_idx
    local ostree_parent_path="$extDeploy/usr/share/deepin-immutable-ctl/state/ostree-parent"
    sys_commit_idx=$(cat "$ostree_parent_path")

    sysroot="$rootmnt"
    ostree_path=/ostree/deploy/deepin/deploy/$sys_commit_idx

    case $extDeploy in
        */ostree/data/*)
            data_layer_mount "$OSTREE" "$sysroot$ostree_path"
            ;;
        *)
            ext_mount "$OSTREE" "$sysroot$ostree_path"
            ;;
    esac
}


# 1. mount root device to $mnt
# 2. mount data device to $mnt/persistent
# 3. mount root and data device as private

#mount /dev/vda4 $mnt
#mount /dev/vda5 $mnt/persistent
#mount --make-private $rootmnt
#mount --make-private $rootmnt/persistent

ostree_mount_root $ostree_arg

# you can do chroot here
