#!/bin/bash
set -e

immutable_adapter_cmd="deepin-immutable-ctl admin exec --"
cmd_args=("$@")
cmd_name=$(basename "$0")

take_over_commands=("apt" "apt-get" "aptitude" "dpkg")

# Function to check if the system is booted in ostree mode
# Returns 1 if in immutable mode, 0 otherwise
function boot_from_ostree {
    command -v deepin-immutable-ctl &> /dev/null && \
    [[ $(deepin-immutable-ctl --immutable-status) =~ "Immutable mode:true" ]] && echo 1 || echo 0
}

function exec_original {
    exec -a "${cmd_name}" "${cmd_name}.real" "${cmd_args[@]}"
}

function exec_immutable {
    IFS=' ' read -ra immutable_cmd_parts <<< ${immutable_adapter_cmd}
    new_cmd_name="${immutable_cmd_parts[0]}"
    new_cmd_args=("${immutable_cmd_parts[@]:1}" ${cmd_name} "${cmd_args[@]}")
    exec -a "${cmd_name}" "${new_cmd_name}" "${new_cmd_args[@]}"
}

function main {
    if [ "$(boot_from_ostree)" -ne 1 ] || [ "${RUN_BY_IMMUTABLE_CTL:-}" == "1" ]; then
        exec_original
    fi

    if [[ " ${take_over_commands[@]} " =~ " ${cmd_name} " ]]; then
        exec_immutable
    else
        exec_original
    fi
}

main "$@"